@@ -1,7 +1,7 @@
 /* Main command processing loop.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -66,6 +66,90 @@ is what most commands require. */
 
 static int perform_wrap;
 
+#ifdef LOGACTIONS
+/* Wrap do_action() and various request* functions to facilitate action logging. */
+
+int do_action_wrapped(buffer *b, action a, int64_t c, char *p);
+static int da_depth = 0;
+static FILE *da_log;
+static buffer *da_prev_b;
+
+bool request_response_wrapper(const buffer *b, const char *prompt, bool default_value) {
+	bool resp = request_response(b, prompt, default_value);
+	if (da_log) {
+		fprintf(da_log,"  %p request_response: %s\n", b, resp ? "true" : "false");
+	}
+	return resp;
+}
+#define request_response request_response_wrapper
+
+char request_char_wrapper(const buffer *b, const char *prompt, const char default_value) {
+	char resp = request_char(b, prompt, default_value);
+	if (da_log) {
+		fprintf(da_log,"  %p request_char: '%c'\n", b, resp);
+	}
+	return resp;
+}
+#define request_char request_char_wrapper
+
+int64_t request_number_wrapper(const buffer *b, const char *prompt, int64_t default_value) {
+	int64_t resp = request_number(b, prompt, default_value);
+	if (da_log) {
+		fprintf(da_log,"  %p request_number: %ld\n", b, resp);
+	}
+	return resp;
+}
+#define request_number request_number_wrapper
+
+char *request_string_wrapper(const buffer *b, const char *prompt, const char *default_string, bool accept_null_string, int completion_type, bool prefer_utf8) {
+	char *resp = request_string(b, prompt, default_string, accept_null_string, completion_type, prefer_utf8);
+	if (da_log) {
+		fprintf(da_log,"  %p request_string: '%s'\n", b, resp ? resp : "<null>");
+	}
+	return resp;
+}
+#define request_string request_string_wrapper
+
+char *request_wrapper(const buffer *b, const char *prompt, const char *default_string, bool alpha_allowed, int completion_type, bool prefer_utf8) {
+	char * resp = request(b, prompt, default_string, alpha_allowed, completion_type, prefer_utf8);
+	if (da_log) {
+		fprintf(da_log,"  %p request: '%s'\n", b, resp ? resp : "<null>");
+	}
+	return resp;
+}
+#define request request_wrapper
+
+char *request_file_wrapper(const buffer *b, const char *prompt, const char *default_name) {
+	char * resp = request_file(b, prompt, default_name);
+	if (da_log) {
+		fprintf(da_log,"  %p request: '%s'\n", b, resp ? resp : "<null>");
+	}
+	return resp;
+}
+#define request_file request_file_wrapper
+int do_action(buffer *b, action a, int64_t c, char *p) {
+	if (!da_log) da_log = fopen("/tmp/ne-actions.log","a");
+	if (da_log) {
+		if (b != da_prev_b && b->filename) {
+			fprintf(da_log, "%p: %s\n", b, b->filename);
+			da_prev_b = b;
+		}
+		fprintf(da_log,"%p%2d %ld,%ld(%ld) %s %ld '%s'\n",
+		                b,
+		                  da_depth++,
+		                      b->cur_line,
+		                          b->cur_pos,
+		                              b->cur_char,
+		                                   command_names[a],
+		                                      c,   p ? p : "<null>");
+		fflush(da_log);
+	}
+	int rc = do_action_wrapped(b, a, c, p);
+	da_depth--;
+	return rc;
+}
+/* End of the insertions to achieve logging of do_action(). */
+#endif
 
 /* This is the dispatcher of all actions that have some effect on the text.
 
@@ -80,8 +164,11 @@ static int perform_wrap;
    somewhere. Though efficient, this has lead to some memory leaks (can you
    find them?). */
 
-
+#ifdef LOGACTIONS
+int do_action_wrapped(buffer *b, action a, int64_t c, char *p) {
+#else
 int do_action(buffer *b, action a, int64_t c, char *p) {
+#endif
 	static char msg[MAX_MESSAGE_SIZE];
 	line_desc *next_ld;
 	HIGHLIGHT_STATE next_line_state;
@@ -99,7 +186,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 	if (b->syn && b->attr_len != -1) {
 		HIGHLIGHT_STATE next_state = parse(b->syn, b->cur_line_desc, b->cur_line_desc->highlight_state, b->encoding == ENC_UTF8);
 		assert(attr_len == b->attr_len);
-		assert(memcmp(attr_buf, b->attr_buf, attr_len) == 0);
+		assert(attr_len == 0 || memcmp(attr_buf, b->attr_buf, attr_len) == 0);
 		assert(memcmp(&next_state, &b->next_state, sizeof next_state) == 0);
 	}
 #endif
@@ -240,33 +327,59 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 	case DELETENEXTWORD_A:
 	case DELETEPREVWORD_A:
+		if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
 		recording = b->recording;
 		b->recording = 0;
 		NORMALIZE(c);
-		delay_update();
+		int marking_t = b->marking;
+		int mark_is_vertical_t = b->mark_is_vertical;
+		b->bookmark[WORDWRAP_BOOKMARK].pos = b->block_start_pos;
+		b->bookmark[WORDWRAP_BOOKMARK].line = b->block_start_line;
+		b->bookmark_mask |= (1 << WORDWRAP_BOOKMARK);
+
 		start_undo_chain(b);
-		for(int64_t i = 0; i < c && !error && !stop; i++) {
-			int marking_t = b->marking;
-			int mark_is_vertical_t = b->mark_is_vertical;
-			b->bookmark[WORDWRAP_BOOKMARK].pos = b->block_start_pos;
-			b->bookmark[WORDWRAP_BOOKMARK].line = b->block_start_line;
-			b->bookmark_mask |= (1 << WORDWRAP_BOOKMARK);
 
-			b->marking = 1;
-			b->mark_is_vertical = 0;
-			b->block_start_line = b->cur_line;
-			b->block_start_pos = b->cur_pos;
+		b->marking = 1;
+		b->mark_is_vertical = 0;
+		b->block_start_line = b->cur_line;
+		b->block_start_pos = b->cur_pos;
 
-			if(!(error = do_action(b, a == DELETENEXTWORD_A ? NEXTWORD_A : PREVWORD_A, 1, NULL))) {
-				if (!(error = erase_block(b))) update_window_lines(b, b->cur_line_desc, b->cur_y, ne_lines - 2, false);
-			}
-			b->bookmark_mask &= ~(1 << WORDWRAP_BOOKMARK);
-			b->block_start_pos = b->bookmark[WORDWRAP_BOOKMARK].pos;
-			b->block_start_line = b->bookmark[WORDWRAP_BOOKMARK].line;
-			b->marking = marking_t;
-			b->mark_is_vertical = mark_is_vertical_t;
+		if (a == DELETEPREVWORD_A) {
+			/* This ugly insertion and deletion of a single character ensures
+				that the cursor ends up here after an undo. */
+			insert_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos, ' ');
+			delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, 1);
+			for(int64_t i = 0; i < c && !(error = search_word(b, -1)) && !stop; i++);
+		} else if (c > 0) {
+			move_to_eow(b);
+			if (b->block_start_line != b->cur_line || b->block_start_pos != b->cur_pos) c--;
+			for(int64_t i = 0; i < c && !(error = search_word(b, 1)) && !stop; i++);
+			if (c) move_to_eow(b);
 		}
-		end_undo_chain(b);
+		const bool line_changed = b->block_start_line != b->cur_line;
+		if (line_changed || b->block_start_pos != b->cur_pos) { /* we moved */
+			b->attr_len = -1;
+			error |= erase_block(b, false);
+			end_undo_chain(b);
+
+			if (line_changed) {
+				if (b->syn) update_syntax_states_delay(b, b->cur_line_desc, NULL);
+				update_window_lines(b, b->cur_line_desc, b->cur_y, ne_lines - 2, false);
+			}
+			else {
+				update_line(b, b->cur_line_desc, b->cur_y, 0, false);
+				if (b->syn) need_attr_update = true;
+			}
+		} else {
+			end_undo_chain(b);
+			if (a == DELETEPREVWORD_A) error |= undo(b);
+		}
+
+		b->bookmark_mask &= ~(1 << WORDWRAP_BOOKMARK);
+		b->block_start_pos = b->bookmark[WORDWRAP_BOOKMARK].pos;
+		b->block_start_line = b->bookmark[WORDWRAP_BOOKMARK].line;
+		b->marking = marking_t;
+		b->mark_is_vertical = mark_is_vertical_t;
 		b->recording = recording;
 		return stop ? STOPPED : error;
 
@@ -349,14 +462,14 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 				b->bookmark[c].cur_y = b->cur_y;
 				b->bookmark_mask |= (1 << c);
 				b->cur_bookmark = c;
-				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c set", c<=MAX_USER_BOOKMARK?'0'+(int)c : '-');
+				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c set", c <= MAX_USER_BOOKMARK?'0' + (int)c : '-');
 				print_message(msg);
 				break;
 			case UNSETBOOKMARK_A:
 				if (! (b->bookmark_mask & (1 << c)))
 					return BOOKMARK_NOT_SET;
 				b->bookmark_mask &= ~(1 << c);
-				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c unset", c<=MAX_USER_BOOKMARK?'0'+(int)c : '-');
+				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c unset", c <= MAX_USER_BOOKMARK?'0' + (int)c : '-');
 				print_message(msg);
 				break;
 			case GOTOBOOKMARK_A:
@@ -378,7 +491,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 					b->bookmark[AUTO_BOOKMARK].cur_y = cur_y;
 					b->bookmark_mask |= 1<<AUTO_BOOKMARK;
 					if (relative) {
-						snprintf(msg, MAX_MESSAGE_SIZE, "At Bookmark %c", c<=MAX_USER_BOOKMARK?'0'+(int)c : '-');
+						snprintf(msg, MAX_MESSAGE_SIZE, "At Bookmark %c", c <= MAX_USER_BOOKMARK?'0' + (int)c : '-');
 						print_message(msg);
 					}
 				}
@@ -388,13 +501,13 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		}
 
 	case GOTOLINE_A:
-		if (c < 0 && (c = request_number(b, "Line", b->cur_line + 1))<0) return NUMERIC_ERROR(c);
+		if (c < 0 && (c = request_number(b, "Line", b->cur_line + 1)) < 0) return NUMERIC_ERROR(c);
 		if (c == 0 || c > b->num_lines) c = b->num_lines;
 		goto_line(b, --c);
 		return OK;
 
 	case GOTOCOLUMN_A:
-		if (c < 0 && (c = request_number(b, "Column", b->cur_x + b->win_x + 1))<0) return NUMERIC_ERROR(c);
+		if (c < 0 && (c = request_number(b, "Column", b->cur_x + b->win_x + 1)) < 0) return NUMERIC_ERROR(c);
 		goto_column(b, c ? --c : 0);
 		return OK;
 
@@ -433,7 +546,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return OK;
 
 	case AUTOMATCHBRACKET_A:
-		if (c < 0 && (c = request_number(b, "Match mode (sum of 0:none, 1:brightness, 2:inverse, 4:bold, 8:underline)", b->opt.automatch))<0||c>15) return ((c) == ABORT ? OK : INVALID_MATCH_MODE);
+		if (c < 0 && (c = request_number(b, "Match mode (sum of 0:none, 1:brightness, 2:inverse, 4:bold, 8:underline)", b->opt.automatch)) < 0 || c > 15) return ((c) == ABORT ? OK : INVALID_MATCH_MODE);
 		b->opt.automatch = c;
 		return OK;
 
@@ -464,7 +577,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 		if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
 
-		if (c < 0 && (c = request_number(b, "Char Code", last_inserted_char))<0) return NUMERIC_ERROR(c);
+		if ((c < 0 || c > MAX_UTF_8) && ((c = request_number(b, "Char Code", last_inserted_char)) < 0 || c > MAX_UTF_8)) return NUMERIC_ERROR(c);
 		if (c == 0) return CANT_INSERT_0;
 
 		if (b->encoding == ENC_ASCII) {
@@ -617,7 +730,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 			if (b->cur_pos < b->cur_line_desc->line_len) {
 				/* Deletion inside a line. */
 				const int old_char = b->encoding == ENC_UTF8 ? utf8char(&b->cur_line_desc->line[b->cur_pos]) : b->cur_line_desc->line[b->cur_pos];
-				const uint32_t old_attr = b->syn ? b->attr_buf[b->cur_pos] : 0;
+				const uint32_t old_attr = b->syn ? b->attr_buf[b->cur_char] : 0;
 				if (b->syn) {
 					/* Invalidate attrs beyond the right window edge. */
 					const int64_t right_char = calc_char_len(b->cur_line_desc, calc_pos(b->cur_line_desc, b->win_x + ne_columns, b->opt.tab_size, b->encoding), b->encoding);
@@ -701,10 +814,8 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 					move_to_sol(b);
 					line_down(b);
 					if (a != -1) goto_pos(b, a);
-
 					if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_line_desc, b->cur_y, 0, false);
 					else scroll_window(b, b->cur_line_desc, b->cur_y, 1);
-
 				}
 				need_attr_update = true;
 			} else end_undo_chain(b);
@@ -815,7 +926,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 	case KEYCODE_A:
 		if (c >= NUM_KEYS) c = -1;
-		if (c < 0 ) {
+		if (c < 0) {
 			print_message(info_msg[PRESS_A_KEY]);
 			do c = get_key_code(); while(c == INVALID_CHAR || c > 0xFF || CHAR_CLASS(c) == IGNORE);
 		}
@@ -832,17 +943,18 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return OK;
 
 	case OPENNEW_A:
-		b = new_buffer();
-		reset_window();
-
-	case OPEN_A:
-		if ((b->is_modified) && !request_response(b, info_msg[THIS_DOCUMENT_NOT_SAVED], false)) {
-			if (a == OPENNEW_A) do_action(b, CLOSEDOC_A, 1, NULL);
-			return ERROR;
+		if (b = new_buffer()) {
+			reset_window();
+		} else {
+			if (p) free(p);
+			return OUT_OF_MEMORY;
 		}
 
+	case OPEN_A:
+		if ((b->is_modified) && !request_response(b, info_msg[THIS_DOCUMENT_NOT_SAVED], false)) return ERROR;
+
 		if (p || (p = request_file(b, "Filename", b->filename))) {
-			static bool dprompt = 0; /* Set to true if we ever respond 'yes' to the prompt. */
+			static bool dprompt = false; /* Set to true if we ever respond 'yes' to the prompt. */
 
 			buffer *dup = get_buffer_named(p);
 
@@ -872,7 +984,10 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 			}
 			free(p);
 		}
-		if (a == OPENNEW_A) do_action(b, CLOSEDOC_A, 1, NULL);
+		if (a == OPENNEW_A) {
+			do_action(b, CLOSEDOC_A, 1, NULL);
+			do_action(cur_buffer, PREVDOC_A, 1, NULL);
+		}
 		return ERROR;
 
 	case ABOUT_A:
@@ -1022,6 +1137,21 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return ERROR;
 
 	case REPEATLAST_A:
+		if (p) {
+			char *q;
+			c = strtoll(p, &q, 0);
+			if (p == q) c = -1;
+			while (*q == ' ') q++;
+			if (*q) {
+				if (     !strcasecmp("f",q) || !strcasecmp("find",q))    b->last_was_replace = false;
+				else if (!strcasecmp("r",q) || !strcasecmp("replace",q)) b->last_was_replace = true;
+				else {
+					free(p);
+					return SYNTAX_ERROR;
+				}
+			}
+			free(p);
+		}
 		if (b->opt.read_only && b->last_was_replace) return DOCUMENT_IS_READ_ONLY;
 		if (!b->find_string) return NO_SEARCH_STRING;
 		if ((b->last_was_replace) && !b->replace_string) return NO_REPLACE_STRING;
@@ -1112,17 +1242,17 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return TAB_SIZE_OUT_OF_RANGE;
 
 	case TURBO_A:
-		if (c < 0 && (c = request_number(b, "Turbo Threshold", turbo))<0) return NUMERIC_ERROR(c);
+		if ((int)c < 0 && (int)(c = request_number(b, "Turbo Threshold", turbo)) < 0) return NUMERIC_ERROR(c);
 		turbo = c;
 		return OK;
 
 	case CLIPNUMBER_A:
-		if (c < 0 && (c = request_number(b, "Clip Number", b->opt.cur_clip))<0) return NUMERIC_ERROR(c);
+		if ((int)c < 0 && (int)(c = request_number(b, "Clip Number", b->opt.cur_clip)) < 0) return NUMERIC_ERROR(c);
 		b->opt.cur_clip = c;
 		return OK;
 
 	case RIGHTMARGIN_A:
-		if (c < 0 && (c = request_number(b, "Right Margin", b->opt.right_margin))<0) return NUMERIC_ERROR(c);
+		if ((int)c < 0 && (int)(c = request_number(b, "Right Margin", b->opt.right_margin)) < 0) return NUMERIC_ERROR(c);
 		b->opt.right_margin = c;
 		return OK;
 
@@ -1386,7 +1516,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
 
 	case COPY_A:
-		if (!(error = print_error((b->mark_is_vertical ? copy_vert_to_clip : copy_to_clip)(b, c < 0 ? b->opt.cur_clip : c, a == CUT_A)))) {
+		if (!(error = print_error((b->mark_is_vertical ? copy_vert_to_clip : copy_to_clip)(b, (int)c < 0 ? b->opt.cur_clip : c, a == CUT_A)))) {
 			b->marking = 0;
 			update_window_lines(b, b->cur_line_desc, b->cur_y, ne_lines - 2, false);
 		}
@@ -1394,7 +1524,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 	case ERASE_A:
 		if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
-		if (!(error = print_error((b->mark_is_vertical ? erase_vert_block : erase_block)(b)))) {
+		if (!(error = print_error((b->mark_is_vertical ? erase_vert_block : erase_block)(b, true)))) {
 			b->marking = 0;
 			update_window_lines(b, b->cur_line_desc, b->cur_y, ne_lines - 2, false);
 		}
@@ -1403,7 +1533,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 	case PASTE_A:
 	case PASTEVERT_A:
 		if (b->opt.read_only) return DOCUMENT_IS_READ_ONLY;
-		if (!(error = print_error((a == PASTE_A ? paste_to_buffer : paste_vert_to_buffer)(b, c < 0 ? b->opt.cur_clip : c)))) update_window_lines(b, b->cur_line_desc, b->cur_y, ne_lines - 2, false);
+		if (!(error = print_error((a == PASTE_A ? paste_to_buffer : paste_vert_to_buffer)(b, (int)c < 0 ? b->opt.cur_clip : c)))) update_window_lines(b, b->cur_line_desc, b->cur_y, ne_lines - 2, false);
 		assert_buffer_content(b);
 		return error ? ERROR : 0;
 
@@ -1507,7 +1637,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 
 									start_undo_chain(b);
 
-									if (b->marking) (b->mark_is_vertical ? erase_vert_block : erase_block)(b);
+									if (b->marking) (b->mark_is_vertical ? erase_vert_block : erase_block)(b, true);
 									error = (b->mark_is_vertical ? paste_vert_to_buffer : paste_to_buffer)(b, INT_MAX);
 									end_undo_chain(b);
 
@@ -1585,6 +1715,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		for(int64_t i = 0; i < c && !(error = paragraph(b)) && !stop; i++);
 		if (stop) error = STOPPED;
 		if (error == STOPPED) reset_window();
+		assert(b->cur_pos >= 0);
 		return print_error(error) ? ERROR : 0;
 
 	case SHIFT_A:
