@@ -1,7 +1,7 @@
 /* Clip handling functions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna 
-   Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -181,7 +181,7 @@ int copy_to_clip(buffer *b, int n, bool cut) {
 				if (cut) {
 					goto_line_pos(b, b->block_start_line, b->block_start_pos);
 					delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, clip_len);
-					update_syntax_and_lines(b, b->cur_line_desc, NULL);
+					update_syntax_states_delay(b, b->cur_line_desc, NULL);
 				}
 
 				if (chaining) end_undo_chain(b);
@@ -241,7 +241,7 @@ int copy_to_clip(buffer *b, int n, bool cut) {
 				set_stream_encoding(cd->cs, b->encoding);
 				assert_clip_desc(cd);
 				if (cut) delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, clip_len);
-				update_syntax_and_lines(b, b->cur_line_desc, NULL);
+				update_syntax_states_delay(b, b->cur_line_desc, NULL);
 				if (chaining) end_undo_chain(b);
 				return OK;
 			}
@@ -262,9 +262,10 @@ int copy_to_clip(buffer *b, int n, bool cut) {
 
 
 
-/* Simply erases a block, without putting it in a clip. Calls update_syntax_and_lines(). */
+/* Simply erases a block, without putting it in a clip. Calls
+   update_syntax_states_delay() if update is true. */
 
-int erase_block(buffer *b) {
+int erase_block(buffer *b, const bool update) {
 	if (!b->marking) return MARK_BLOCK_FIRST;
 	if (b->block_start_line >= b->num_lines) return MARK_OUT_OF_BUFFER;
 
@@ -323,7 +324,7 @@ int erase_block(buffer *b) {
 
 	delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, erase_len - 1);
 	if (chaining) end_undo_chain(b);
-	update_syntax_and_lines(b, b->cur_line_desc, NULL);
+	if (update) update_syntax_states_delay(b, b->cur_line_desc, NULL);
 	return OK;
 }
 
@@ -351,7 +352,7 @@ int paste_to_buffer(buffer *b, int n) {
 		end_undo_chain(b);
 
 		assert(ld == b->cur_line_desc);
-		update_syntax_and_lines(b, ld, end_ld);
+		update_syntax_states_delay(b, ld, end_ld);
 		return OK;
 	}
 
@@ -427,7 +428,7 @@ int copy_vert_to_clip(buffer *b, int n, bool cut) {
 				set_stream_encoding(cd->cs, b->encoding);
 				assert_clip_desc(cd);
 				if (cut) {
-					update_syntax_and_lines(b, (line_desc *)ld->ld_node.next, b->cur_line_desc);
+					update_syntax_states_delay(b, (line_desc *)ld->ld_node.next, b->cur_line_desc);
 					goto_line_pos(b, min(b->block_start_line, b->cur_line), min(b->block_start_pos, cur_pos));
 					end_undo_chain(b);
 				}
@@ -468,7 +469,7 @@ int copy_vert_to_clip(buffer *b, int n, bool cut) {
 				set_stream_encoding(cd->cs, b->encoding);
 				assert_clip_desc(cd);
 				if (cut) {
-					update_syntax_and_lines(b, b->cur_line_desc, (line_desc *)ld->ld_node.prev);
+					update_syntax_states_delay(b, b->cur_line_desc, (line_desc *)ld->ld_node.prev);
 					goto_line_pos(b, min(b->block_start_line, b->cur_line), min(b->block_start_pos, cur_pos));
 					end_undo_chain(b);
 				}
@@ -487,10 +488,10 @@ int copy_vert_to_clip(buffer *b, int n, bool cut) {
 	return OK;
 }
 
-/* Simply erases a vertical block, without putting it in a clip. Calls update_syntax_and_lines(). */
-
-int erase_vert_block(buffer *b) {
+/* Simply erases a vertical block, without putting it in a clip. Calls
+   update_syntax_states_delay() if update is true. */
 
+int erase_vert_block(buffer *b, const bool update) {
 
 	if (!b->marking) return MARK_BLOCK_FIRST;
 	if (b->block_start_line >= b->num_lines) return MARK_OUT_OF_BUFFER;
@@ -524,7 +525,7 @@ int erase_vert_block(buffer *b) {
 			delete_stream(b, ld, i, start_pos, len);
 			ld = (line_desc *)ld->ld_node.prev;
 		}
-		update_syntax_and_lines(b, (line_desc *)ld->ld_node.next, b->cur_line_desc);
+		if (update) update_syntax_states_delay(b, (line_desc *)ld->ld_node.next, b->cur_line_desc);
 	}
 	else {
 		for(int64_t i = y; i <= b->block_start_line; i++) {
@@ -533,7 +534,7 @@ int erase_vert_block(buffer *b) {
 			delete_stream(b, ld, i, start_pos, len);
 			ld = (line_desc *)ld->ld_node.next;
 		}
-		update_syntax_and_lines(b, b->cur_line_desc, (line_desc *)ld->ld_node.prev);
+		if (update) update_syntax_states_delay(b, b->cur_line_desc, (line_desc *)ld->ld_node.prev);
 	}
 
 	end_undo_chain(b);
@@ -595,7 +596,7 @@ int paste_vert_to_buffer(buffer *b, int n) {
 	}
 	
 	end_undo_chain(b);
-	update_syntax_and_lines(b, b->cur_line_desc, ld);
+	update_syntax_states_delay(b, b->cur_line_desc, ld);
 	return OK;
 }
 
