@@ -1,7 +1,7 @@
 /* Command table manipulation functions and vectors.
 
-   Copyright (C) 1993-1998 Sebastiano Vigna 
-   Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna
+   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -158,7 +158,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(RECORD        ),                           IS_OPTION | DO_NOT_RECORD                   },
 	{ NAHL(REDO          ),0                                                                      },
 	{ NAHL(REFRESH       ), NO_ARGS                                                               },
-	{ NAHL(REPEATLAST    ),0                                                                      },
+	{ NAHL(REPEATLAST    ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(REPLACE       ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(REPLACEALL    ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
 	{ NAHL(REPLACEONCE   ),           ARG_IS_STRING |                             EMPTY_STRING_OK },
@@ -690,7 +690,7 @@ char *find_key_strokes(int c, int n) {
 
 char *bound_keys_string(int c) {
 	char *key_strokes = find_key_strokes(c, 9);
-	char *str=NULL, *p;
+	char *str=NULL;
 	if (key_strokes) {
 		if ((str = malloc(strlen(key_strokes) + 16)))
 			strcat(strcpy(str, "Bound keys(s): "), key_strokes);
