@@ -1,7 +1,7 @@
 /* Display handling functions with optional update delay
 
    Copyright (C) 1993-1998 Sebastiano Vigna 
-   Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -94,7 +94,6 @@ void update_syntax_states(buffer *b, int row, line_desc *ld, line_desc *end_ld)
  		bool got_end_ld = end_ld == NULL;
 		bool invalidate_attr_buf = false;
 		HIGHLIGHT_STATE next_line_state = b->attr_len < 0 ? parse(b->syn, ld, ld->highlight_state, b->encoding == ENC_UTF8) : b->next_state;
-
 		assert(b->attr_len < 0 || b->attr_len == calc_char_len(ld, ld->line_len, b->encoding));
 
 		for(;;) {
@@ -105,7 +104,6 @@ void update_syntax_states(buffer *b, int row, line_desc *ld, line_desc *end_ld)
 			/* We update lines until next_line_state is equal to our current highlight_state, but we go until
 			   end_ld if it is not NULL. In any case, we bail out at the end of the file. */
 			if ((highlight_cmp(&ld->highlight_state, &next_line_state) && got_end_ld) || !ld->ld_node.next) break;
-
 			if (row >= 0) {
 				row++;
 				if (row < ne_lines - 1) {
@@ -183,9 +181,8 @@ void output_line_desc(const int row, const int col, const line_desc *ld, const i
 
 		if (*s == '\t') {
 			const int tab_width = tab_size - curr_col % tab_size;
-			int i;
 
-			for(i = 0; i < tab_width; i++)
+			for(int i = 0; i < tab_width; i++)
 				if (curr_col + i >= from_col && curr_col + i < from_col + num_cols) {
 					move_cursor(row, output_col + i);
 					output_char(' ', attr ? attr[attr_pos] : 0, false);
@@ -196,7 +193,7 @@ void output_line_desc(const int row, const int col, const line_desc *ld, const i
 		else {
 			const int c_width = output_width(c);
 
-			if (output_col >= col || output_col + c_width > col && output_col >= 0) {
+			if (output_col >= col) {
 				if (output_col + c_width <= ne_columns) {
 					if (attr) {
 						/* In the case of a differential update, we output only
@@ -218,6 +215,15 @@ void output_line_desc(const int row, const int col, const line_desc *ld, const i
 					output_spaces(ne_columns - output_col, attr ? &attr[attr_pos] : NULL);
 				}
 			}
+			else if (output_col + c_width > col) {
+				/* The caracter is only partially displayed. We can only output spaces. */
+				const int output_width = output_col + c_width - col;
+				for(int i = 0; i < output_width; i++) {
+					move_cursor(row, col + i);
+					output_char(' ', attr ? attr[attr_pos] : 0, false);
+				}
+			}
+
 			curr_col += c_width;
 		}
 		s += c_len;
@@ -251,7 +257,7 @@ void output_line_desc(const int row, const int col, const line_desc *ld, const i
 
    After a call to this function with argument b->cur_line_desc,
    b->attr_buf will be updated so to contain again the currently displayed
-   attributes, unless no refresh has been performed, in which base
+   attributes, unless no refresh has been performed, in which case
    b->attr_len will be set to -1.
 */
 
@@ -288,7 +294,8 @@ void update_line(buffer * const b, line_desc * const ld, const int row, const in
 			/* If we updated current line, we update the local attribute buffer. */
 			b->next_state = next_state;
 			ensure_attr_buf(b, attr_len);
-			memcpy(b->attr_buf, attr_buf, (b->attr_len = attr_len) * sizeof *b->attr_buf);
+			// This test is necessary to avoid warnings from -fsanitize
+			if (b->attr_len = attr_len) memcpy(b->attr_buf, attr_buf, attr_len * sizeof *b->attr_buf);
 		}
 	}
 	else output_line_desc(row, from_col, ld, from_col + b->win_x, ne_columns - from_col, b->opt.tab_size, cleared_at_end, b->encoding == ENC_UTF8, NULL, NULL, 0);
@@ -346,7 +353,7 @@ void update_window(buffer * const b) {
    a call to delay_update(). This is mainly written to fix the screen
    state after a block operation. */
 
-void update_syntax_and_lines(buffer *b, line_desc *start_ld, line_desc *end_ld) {
+void update_syntax_states_delay(buffer *b, line_desc *start_ld, line_desc *end_ld) {
 	delay_update();
 
 	if (b->syn) {
@@ -373,20 +380,21 @@ void update_syntax_and_lines(buffer *b, line_desc *start_ld, line_desc *end_ld)
    taken into consideration. */
 
 void update_deleted_char(buffer * const b, const int c, const int a, line_desc * const ld, int64_t pos, int64_t attr_pos, const int line, const int x) {
-	if (b->syn) {
-		assert(b->attr_len >= 0);
-		assert(b->attr_len - 1 == calc_char_len(ld, ld->line_len, b->encoding));
-		memmove(b->attr_buf + attr_pos, b->attr_buf + attr_pos + 1, (--b->attr_len - attr_pos) * sizeof *b->attr_buf);
-	}
-
 	if (++updated_lines > TURBO) window_needs_refresh = true;
 
 	if (window_needs_refresh) {
 		if (line < first_line) first_line = line;
 		if (line > last_line) last_line = line;
+		b->attr_len = -1;
 		return;
 	}
 
+	if (b->syn) {
+		assert(b->attr_len >= 0);
+		assert(b->attr_len > attr_pos);
+		memmove(b->attr_buf + attr_pos, b->attr_buf + attr_pos + 1, (--b->attr_len - attr_pos) * sizeof *b->attr_buf);
+	}
+
 	if (pos > ld->line_len || (pos == ld->line_len && ((c == '\t' || c == ' ') && !a))) return;
 
 	move_cursor(line, x);
@@ -449,6 +457,15 @@ void update_deleted_char(buffer * const b, const int c, const int a, line_desc *
 void update_inserted_char(buffer * const b, const int c, line_desc * const ld, const int64_t pos, const int64_t attr_pos, const int line, const int x) {
 	assert(pos < ld->line_len);
 
+	if (++updated_lines > TURBO) window_needs_refresh = true;
+
+	if (window_needs_refresh) {
+		if (line < first_line) first_line = line;
+		if (line > last_line) last_line = line;
+		b->attr_len = -1;
+		return;
+	}
+
 	const uint32_t * const attr = b->syn ? &attr_buf[attr_pos] : NULL;
 
 	if (b->syn) {
@@ -461,14 +478,6 @@ void update_inserted_char(buffer * const b, const int c, line_desc * const ld, c
 		b->attr_buf[attr_pos] = *attr;
 	}
 
-	if (++updated_lines > TURBO) window_needs_refresh = true;
-
-	if (window_needs_refresh) {
-		if (line < first_line) first_line = line;
-		if (line > last_line) last_line = line;
-		return;
-	}
-
 	move_cursor(line, x);
 
 	const int c_len = b->encoding == ENC_UTF8 ? utf8seqlen(c) : 1;
@@ -521,6 +530,15 @@ void update_overwritten_char(buffer * const b, const int old_char, const int new
 	assert(ld != NULL);
 	assert(pos < ld->line_len);
 
+	if (++updated_lines > TURBO) window_needs_refresh = true;
+
+	if (window_needs_refresh) {
+		if (line < first_line) first_line = line;
+		if (line > last_line) last_line = line;
+		b->attr_len = -1;
+		return;
+	}
+
 	const uint32_t * const attr = b->syn ? &attr_buf[attr_pos] : NULL;
 
 	if (b->syn) {
@@ -531,14 +549,6 @@ void update_overwritten_char(buffer * const b, const int old_char, const int new
 		b->attr_buf[attr_pos] = *attr;
 	}
 
-	if (++updated_lines > TURBO) window_needs_refresh = true;
-
-	if (window_needs_refresh) {
-		if (line < first_line) first_line = line;
-		if (line > last_line) last_line = line;
-		return;
-	}
-
 	const int old_width = old_char == '\t' ? b->opt.tab_size - x % b->opt.tab_size : output_width(old_char);
 	const int new_width = new_char == '\t' ? b->opt.tab_size - x % b->opt.tab_size : output_width(new_char);
 
@@ -730,8 +740,9 @@ void ensure_attributes(buffer *b) {
 void store_attributes(buffer *b, line_desc *ld) {
 	b->next_state = parse(b->syn, ld, ld->highlight_state, b->encoding == ENC_UTF8);
 	assert(calc_char_len(ld, ld->line_len, b->encoding) == attr_len);
+	// This test is necessary to avoid warnings from -fsanitize
 	ensure_attr_buf(b, attr_len);
-	memcpy(b->attr_buf, attr_buf, (b->attr_len = attr_len) * sizeof *b->attr_buf);
+	if (b->attr_len = attr_len) memcpy(b->attr_buf, attr_buf, attr_len * sizeof *b->attr_buf);
 }
 
 /* (Un)highlights (depending on the value of show) the bracket matching
