@@ -1,7 +1,7 @@
 /* Extern's for names and abbreviations of all the commands.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -30,6 +30,8 @@ struct e2s {
 
 static struct e2s const e2s[] = {
 	{ "1", "troff" },
+	{ "F", "fortran" },
+	{ "F90", "fortran" },
 	{ "adb", "ada" },
 	{ "ads", "ada" },
 	{ "bash", "sh" },
@@ -42,10 +44,12 @@ static struct e2s const e2s[] = {
 	{ "cc", "c" },
 	{ "cob", "cobol" },
 	{ "cpp", "c" },
+	{ "cxx", "c" },
 	{ "dtx", "tex" },
 	{ "el", "lisp" },
 	{ "eps", "ps" },
 	{ "f", "fortran" },
+	{ "f90", "fortran" },
 	{ "for", "fortran" },
 	{ "h", "c" },
 	{ "h++", "c" },
@@ -83,7 +87,8 @@ static struct e2s const e2s[] = {
 	{ "vhd", "verilog" },
 	{ "xsd", "xml" },
 	{ "y", "c" },
-	{ "yacc", "c" }
+	{ "yacc", "c" },
+	{ "yml", "yaml" }
 
 };
 
