@@ -338,22 +338,25 @@ const char * const DELETELINE_HELP[9] = {
   "respect to 'InsertLine'.",
 };
 
-const char * const DELETENEXTWORD_HELP[6] = {
+const char * const DELETENEXTWORD_HELP[8] = {
   "Syntax: DeleteNextWord [N]",
   "Abbreviation: DNW",
   "",
-  "deletes text from the current position to the next word N times.  If the",
-  "optional N argument is not specified, it is assumed to be one.",
+  "deletes text from the current position to the N'th next end-of-word If",
+  "the optional N argument is not specified, it is assumed to be one (in",
+  "which case, if the cursor is in the middle of a word the effect is just",
+  "to delete to the end of that word).",
 };
 
-const char * const DELETEPREVWORD_HELP[8] = {
+const char * const DELETEPREVWORD_HELP[9] = {
   "Syntax: DeletePrevWord [N]",
   "Abbreviation: DPW",
   "",
-  "deletes text from the current position to the first character of the",
-  "previous word N times.  If the optional N argument is not specified, it",
-  "is assumed to be one (in which case, if the cursor is in the middle of a",
-  "word the effect is just to delete to the start of that word).",
+  "deletes text from the current position through the first character of",
+  "the N'th previous start-of-word.  If the optional N argument is not",
+  "specified, it is assumed to be one (in which case, if the cursor is in",
+  "the middle of a word the effect is just to delete to the start of that",
+  "word).",
 };
 
 const char * const DELTABS_HELP[9] = {
@@ -1338,13 +1341,14 @@ const char * const REFRESH_HELP[11] = {
   "has changed, and will modify the display to take that into account.",
 };
 
-const char * const REPEATLAST_HELP[30] = {
-  "Syntax: RepeatLast [TIMES]",
+const char * const REPEATLAST_HELP[31] = {
+  "Syntax: RepeatLast [TIMES] [F|Find|R|Replace]",
   "Abbreviation: RL",
   "",
   "repeats for the given number of times the last find or replace operation",
   "(with replace we mean here a single replace, even if the last 'Replace'",
-  "operation ended with a global substitution).",
+  "operation ended with a global substitution).  If you don't specify",
+  "either FIND or REPLACE, it will repeat whichever one was last performed.",
   "",
   "   If any find or replace operation runs into the end of the document",
   "(in the direction indicated by the 'SearchBack' flag) and stops, then",
