@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 
 use strict;
-$| = 1;             
+$| = 1;
 
 # Todd_Lewis@unc.edu
 #
@@ -12,8 +12,8 @@ $| = 1;
 # (and probably other versions too).
 
 my $copyright =
-  qq[	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
+  qq[	Copyright (C) 1993-1998 Sebastiano Vigna
+	Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -60,7 +60,7 @@ foreach ( @infofiles )
     # as well as multiple lines like this:
     #    See *Note Key
     #    Bindings::.
-    
+
     open INFO, $_ or die "Can't read command names from '$_'.";
     my @line = <INFO>;
     close INFO;
@@ -90,9 +90,9 @@ foreach ( @infofiles )
                       $state = 'searching';
                       next;
                   }
-    
+
               # print "---2 read \"$_\"\n";
-    
+
               if ( m/^(Abbreviation: )['`](.+)'/ )
                   {
                     $commands{$command}->{"abbr"} = "$2";
@@ -132,9 +132,9 @@ sub hash
       }
     return $h;
   }
-    
+
 # To this purpose, two different hash tables are built for command names
-# and for abbreviations. This helps the distribution of the string to be 
+# and for abbreviations. This helps the distribution of the string to be
 # as random as possible.
 #
 # Using a sieve, the prime numbers up to $max_hash_table_size are generated.
@@ -182,9 +182,9 @@ sub gen_hash_tables
 
         @long_hash_table  = (0) x $hash_table_size;
         @short_hash_table = (0) x $hash_table_size;
-        
+
         $cmd_index = 0;
-        
+
         foreach $cmd ( sort keys %commands )
           {
             $long  = hash($commands{$cmd}->{"cmd" });
@@ -194,10 +194,10 @@ sub gen_hash_tables
             $long_hash_table[ $long ] = $cmd_index;
             $short_hash_table[$short] = $cmd_index;
           }
-          
+
         # If we get here, we've got hash tables with no conflicts.
         # Declare victory and move on...
-        print "$0: HASH_TABLE_SIZE = $hash_table_size\n"; 
+        print "$0: HASH_TABLE_SIZE = $hash_table_size\n";
         return 1;
       }
     # If we get here, we failed to generate hash tables with no conflicts.
@@ -374,7 +374,7 @@ _EOT_
     $body_names .= "  ${uccmd}_NAME,";
     $body_names .= "\n" if ( ( my @x = ($body_names =~ m/,/g)) % 4 == 0 );
   }
-  
+
 sub tail_names_c
   {
     print NAMES_C <<"_EOT_";
@@ -383,6 +383,10 @@ sub tail_names_c
 const char PLAYONCE_ABBREV[] = "PL 1";
 const char MIDDLEVIEW_ABBREV[] = "AV M";
 const char SHIFTLEFT_ABBREV[] = "SH <";
+#ifdef ALTPAGING
+const char SHIFTSPACELEFT_ABBREV[] = "SH <S";
+const char SHIFTSPACERIGHT_ABBREV[] = "SH >S";
+#endif
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
@@ -420,7 +424,7 @@ extern const char ${uccmd}_ABBREV[];
 
 _EOT_
   }
-  
+
 sub tail_names_h
   {
     print NAMES_H <<"_EOT_";
@@ -429,6 +433,10 @@ sub tail_names_h
 extern const char PLAYONCE_ABBREV[];
 extern const char MIDDLEVIEW_ABBREV[];
 extern const char SHIFTLEFT_ABBREV[];
+#ifdef ALTPAGING
+extern const char SHIFTSPACELEFT_ABBREV[];
+extern const char SHIFTSPACERIGHT_ABBREV[];
+#endif
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
@@ -470,7 +478,7 @@ sub body_help_c
       }
     print HELP_C "};\n\n";
   }
-  
+
 sub tail_help_c
   {
     print HELP_C <<"_EOT_";
@@ -505,7 +513,7 @@ sub body_help_h
 extern const char * const ${uccmd}_HELP[ $len ];
 _EOT_
   }
-  
+
 sub tail_help_h
   {
     print HELP_H <<"_EOT_";
@@ -541,7 +549,7 @@ sub body_enums_h
     $body_enums .= "  ${uccmd}_A,";  # Count the commas, add '\n' if we've got multiple of 4.
     $body_enums .= "\n" if ( (my @x = ($body_enums =~ m/,/g)) % 4 == 0 );
   }
-  
+
 sub tail_enums_h
   {
     print ENUMS_H <<"_EOT_";
@@ -573,7 +581,7 @@ sub body_hash_h
   {
     # Nothing to do here.
   }
-  
+
 sub tail_hash_h
   {
     my $max_command_width = 0;
@@ -599,7 +607,7 @@ names with the string requester. For example, 18 would allow four columns
 on an 80x25 screen.
 */
 
-#define MAX_COMMAND_WIDTH		$max_command_width
+#define MAX_COMMAND_WIDTH  $max_command_width
 
 
 /* This file was automatically generated by $0. */
@@ -640,7 +648,7 @@ sub body_hash_c
   {
     # Nothing to do here.
   }
-  
+
 sub tail_hash_c
   {
     print HASH_C "const unsigned char hash_table[HASH_TABLE_SIZE] = {\n";
@@ -649,16 +657,16 @@ sub tail_hash_c
         print HASH_C "  $long_hash_table[$i],";
         print HASH_C "\n" if ( $i % 20 == 0 );
       }
-    print HASH_C "};\n\n"; 
-    
+    print HASH_C "};\n\n";
+
     print HASH_C "const unsigned char short_hash_table[HASH_TABLE_SIZE] = {\n";
     for (my $i=0; $i<$hash_table_size; $i++)
       {
         print HASH_C "  $short_hash_table[$i],";
         print HASH_C "\n" if ( $i % 20 == 0 );
       }
-    print HASH_C "};\n\n"; 
-    
+    print HASH_C "};\n\n";
+
     print HASH_C <<"_EOT_";
 /* This file was automatically generated by $0. */
 _EOT_
