@@ -1,7 +1,7 @@
 /* Extern's for names and abbreviations of all the commands.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1993-1998 Sebastiano Vigna
+	Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -229,6 +229,9 @@ extern const char MOVESOL_ABBREV[];
 extern const char MOVETOS_NAME[];
 extern const char MOVETOS_ABBREV[];
 
+extern const char NAMECONVERT_NAME[];
+extern const char NAMECONVERT_ABBREV[];
+
 extern const char NEWDOC_NAME[];
 extern const char NEWDOC_ABBREV[];
 
@@ -438,6 +441,10 @@ extern const char WORDWRAP_ABBREV[];
 extern const char PLAYONCE_ABBREV[];
 extern const char MIDDLEVIEW_ABBREV[];
 extern const char SHIFTLEFT_ABBREV[];
+#ifdef ALTPAGING
+extern const char SHIFTSPACELEFT_ABBREV[];
+extern const char SHIFTSPACERIGHT_ABBREV[];
+#endif
 
 /* This is the NULL-terminated, ordered list of names, useful for help etc. */
 
