@@ -1,7 +1,7 @@
 /* Navigation functions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -684,7 +684,11 @@ parameter -1. */
 void goto_line_pos(buffer * const b, const int64_t n, const int64_t pos) {
 	b->y_wanted = 0;
 
-	if (n >= b->num_lines || n == b->cur_line && (pos == -1 || pos == b->cur_pos)) return;
+	if (n >= b->num_lines || n == b->cur_line && pos == b->cur_pos) return;
+	if (n == b->cur_line && pos == -1) {
+		resync_pos(b);
+		return;
+	}
 
 	line_desc *ld;
 
