@@ -1,7 +1,7 @@
 /* Function prototypes
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2017 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2018 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -72,10 +72,10 @@ void free_clip_desc(clip_desc *cd);
 int is_encoding_neutral(clip_desc *cd);
 clip_desc *get_nth_clip(int n);
 int copy_to_clip(buffer *b, int n, bool cut);
-int erase_block(buffer *b);
+int erase_block(buffer *b, const bool update);
 int paste_to_buffer(buffer *b, int n);
 int copy_vert_to_clip(buffer *b, int n, bool cut);
-int erase_vert_block(buffer *b);
+int erase_vert_block(buffer *b, const bool update);
 int paste_vert_to_buffer(buffer *b, int n);
 int load_clip(int n, const char *name, bool preserve_cr, bool binary);
 int save_clip(int n, const char *name, bool CRLF, bool binary);
@@ -104,7 +104,7 @@ void delay_update();
 void output_line_desc(int row, int col, const line_desc *ld, int64_t start, int64_t len, int tab_size, bool cleared_at_end, bool utf8, const uint32_t * const attr, const uint32_t * const diff, const int64_t diff_size);
 void update_line(buffer *b, line_desc *ld, int n, int64_t start_x, bool cleared_at_end);
 void update_window_lines(buffer *b, line_desc *ld, int start_line, int end_line, bool doit);
-void update_syntax_and_lines(buffer *b, line_desc *start_ld, line_desc *end_ld);
+void update_syntax_states_delay(buffer *b, line_desc *start_ld, line_desc *end_ld);
 void update_window(buffer *b);
 void update_deleted_char(buffer *b, int c, int a, line_desc *ld, int64_t pos, int64_t attr_pos, int line, int x);
 void update_inserted_char(buffer *b, int c, line_desc *ld, int64_t pos, int64_t attr_pos, int line, int x);
